@@ -3,40 +3,40 @@ from ajax_helpers.html_include import SourceBase
 
 
 class Dot(SourceBase):
-    static_path = '/advanced_report_builder/dot/'
+    static_path = 'advanced_report_builder/dot/'
     js_filename = 'doT.js'
 
 
 class JQueryExtendext(SourceBase):
-    static_path = '/advanced_report_builder/jquery_extendext/'
+    static_path = 'advanced_report_builder/jquery_extendext/'
     js_filename = 'jQuery.extendext.js'
 
 
 class QueryBuilder(SourceBase):
-    static_path = '/advanced_report_builder/query_builder/'
+    static_path = 'advanced_report_builder/query_builder/'
     css_filename = 'query-builder.default.css'
     js_filename = 'query-builder.min.js'
 
 
 class DashboardInclude(SourceBase):
-    static_path = '/advanced_report_builder/dashboard/'
+    static_path = 'advanced_report_builder/dashboard/'
     js_filename = 'dashboard.js'
 
 
 class ChartJS(SourceBase):
-    static_path = '/advanced_report_builder/chart-js/'
+    static_path = 'advanced_report_builder/chart-js/'
     js_filename = ['chart.min.js',
                    'chartjs-adapter-moment.min.js',  # this does require moment however it's already included
                    'chartjs-plugin-datalabels.min.js']
 
 
 class D3(SourceBase):
-    static_path = '/advanced_report_builder/d3/'
+    static_path = 'advanced_report_builder/d3/'
     js_filename = ['d3.min.js']
 
 
 class D3Funnel(SourceBase):
-    static_path = '/advanced_report_builder/d3-funnel/'
+    static_path = 'advanced_report_builder/d3-funnel/'
     js_filename = ['d3-funnel.min.js']
 
 
