@@ -4,6 +4,7 @@ import operator
 from datetime import datetime, timedelta
 from functools import reduce
 
+from crispy_forms.bootstrap import StrictButton
 from crispy_forms.layout import Div
 from date_offset.monthdelta import MonthDelta
 from django.core.exceptions import FieldDoesNotExist
@@ -35,6 +36,7 @@ from advanced_report_builder.views.charts_base import ChartBaseView, ChartBaseFi
 from advanced_report_builder.views.datatables.modal import TableFieldModal, TableFieldForm
 from advanced_report_builder.views.datatables.utils import TableUtilsMixin
 from advanced_report_builder.views.modals_base import QueryBuilderModalBaseMixin, QueryBuilderModalBase
+from advanced_report_builder.views.query_modal.mixin import MultiQueryModalMixin
 
 
 class BarChartView(ChartBaseView):
@@ -72,11 +74,18 @@ class BarChartView(ChartBaseView):
             date_format = DATE_FORMAT_TYPES_DJANGO_FORMAT[default_format_type]
         return date_format
 
-    def set_extra_number_field_kwargs(self, data_attr, options, multiple_index):
-        negative_bar_colour = data_attr.get('negative_bar_colour') or '801C70'
-        positive_bar_colour = data_attr.get('positive_bar_colour') or '801C70'
-        negative_bar_colour = self.add_colour_offset(negative_bar_colour, multiple_index=multiple_index)
-        positive_bar_colour = self.add_colour_offset(positive_bar_colour, multiple_index=multiple_index)
+    def set_extra_number_field_kwargs(self, data_attr, options, multiple_index, additional_options):
+        negative_bar_colour = None
+        positive_bar_colour = None
+        if additional_options is not None:
+            negative_bar_colour = additional_options.get('negative_colour')
+            positive_bar_colour = additional_options.get('positive_colour')
+        if negative_bar_colour is None or negative_bar_colour == '':
+            negative_bar_colour = data_attr.get('negative_bar_colour') or '801C70'
+            negative_bar_colour = self.add_colour_offset(negative_bar_colour, multiple_index=multiple_index)
+        if positive_bar_colour is None or positive_bar_colour == '':
+            positive_bar_colour = data_attr.get('positive_bar_colour') or '801C70'
+            positive_bar_colour = self.add_colour_offset(positive_bar_colour, multiple_index=multiple_index)
 
         options.update({'colours': {'negative': negative_bar_colour,
                                     'positive': positive_bar_colour}})
@@ -141,11 +150,12 @@ class BarChartView(ChartBaseView):
         return start_field_name
 
 
-class BarChartModal(QueryBuilderModalBase):
+class BarChartModal(MultiQueryModalMixin, QueryBuilderModalBase):
     template_name = 'advanced_report_builder/charts/modal.html'
     process = PROCESS_EDIT_DELETE
     permission_delete = PERMISSION_OFF
     model = BarChartReport
+    show_order_by = False
     widgets = {'positive_bar_colour': ColourPickerWidget,
                'negative_bar_colour': ColourPickerWidget,
                'stacked': RBToggle,
@@ -175,7 +185,7 @@ class BarChartModal(QueryBuilderModalBase):
                    'breakdown_fields']
 
     def form_setup(self, form, *_args, **_kwargs):
-        if 'data' in _kwargs:
+        if 'data' in _kwargs and len(_kwargs['data']) > 0:
             date_field = _kwargs['data'].get('date_field')
             end_date_field = _kwargs['data'].get('end_date_field')
             report_type_id = _kwargs['data'].get('report_type')
@@ -199,7 +209,6 @@ class BarChartModal(QueryBuilderModalBase):
 
         form.fields['notes'].widget.attrs['rows'] = 3
 
-        self.add_query_data(form)
         url = reverse('advanced_report_builder:bar_chart_field_modal',
                       kwargs={'slug': 'selector-99999-data-FIELD_INFO-report_type_id-REPORT_TYPE_ID'})
 
@@ -219,33 +228,34 @@ class BarChartModal(QueryBuilderModalBase):
                         BarChartReport.DATE_FIELD_RANGE: ('html', 'Start date field')}}
         ])
 
-        return ('name',
-                'notes',
-                'report_type',
-                'report_tags',
-                'bar_chart_orientation',
-                'axis_scale',
-                'axis_value_type',
-                'date_field_type',
-                'date_field',
-                'end_date_field',
-                FieldEx('fields',
-                        template='advanced_report_builder/select_column.html',
-                        extra_context={'select_column_url': url,
-                                       'command_prefix': ''}),
-                'x_label',
-                'y_label',
-                'stacked',
-                'show_totals',
-                'show_blank_dates',
-                'show_breakdown',
-                FieldEx('breakdown_fields',
-                        template='advanced_report_builder/select_column.html',
-                        extra_context={'select_column_url': url_breakdown,
-                                       'command_prefix': 'breakdown_'}),
-                FieldEx('query_data',
-                        template='advanced_report_builder/query_builder.html'),
-                )
+        fields = ['name',
+                  'notes',
+                  'report_type',
+                  'report_tags',
+                  'bar_chart_orientation',
+                  'axis_scale',
+                  'axis_value_type',
+                  'date_field_type',
+                  'date_field',
+                  'end_date_field',
+                  FieldEx('fields',
+                          template='advanced_report_builder/select_column.html',
+                          extra_context={'select_column_url': url,
+                                         'command_prefix': ''}),
+                  'x_label',
+                  'y_label',
+                  'stacked',
+                  'show_totals',
+                  'show_blank_dates',
+                  'show_breakdown',
+                  FieldEx('breakdown_fields',
+                          template='advanced_report_builder/select_column.html',
+                          extra_context={'select_column_url': url_breakdown,
+                                         'command_prefix': 'breakdown_'})
+                  ]
+        if self.object.id:
+            self.add_extra_queries(form=form, fields=fields)
+        return fields
 
     def select2_date_field(self, **kwargs):
         return self.get_fields_for_select2(field_type='date',
@@ -306,21 +316,36 @@ class BarChartFieldForm(ChartBaseFieldForm):
                 self.fields['filter'].initial = decode_attribute(data_attr['filter'])
 
         self.fields['multiple_columns'] = BooleanField(required=False, widget=RBToggle())
-
+        self.fields['append_column_title'] = BooleanField(required=False, widget=RBToggle())
         report_builder_class = get_report_builder_class(model=base_model,
                                                         report_type=report_type)
 
-        fields = []
+        self.setup_colour_field(form_fields=self.fields,
+                                base_model=base_model,
+                                report_builder_class=report_builder_class,
+                                name='positive_colour_field',
+                                data_attr=data_attr)
+        self.setup_colour_field(form_fields=self.fields,
+                                base_model=base_model,
+                                report_builder_class=report_builder_class,
+                                name='negative_colour_field',
+                                data_attr=data_attr)
+
+        multiple_column_field = []
         self._get_query_builder_foreign_key_fields(base_model=base_model,
                                                    report_builder_class=report_builder_class,
-                                                   fields=fields)
-
-        self.fields['multiple_column_field'] = ChoiceField(choices=fields, required=False)
+                                                   fields=multiple_column_field)
+        self.fields['multiple_column_field'] = ChoiceField(choices=multiple_column_field,
+                                                           required=False,
+                                                           widget=Select2())
 
         if data_attr.get('multiple_columns') == '1':
             self.fields['multiple_columns'].initial = True
             self.fields['multiple_column_field'].initial = data_attr.get('multiple_column_field')
 
+            if data_attr.get('append_column_title') == '1':
+                self.fields['append_column_title'].initial = True
+
         super().setup_modal(*args, **kwargs)
 
     def get_additional_attributes(self):
@@ -339,7 +364,18 @@ class BarChartFieldForm(ChartBaseFieldForm):
 
             if self.cleaned_data['multiple_columns']:
                 attributes.append('multiple_columns-1')
-                attributes.append(f'multiple_column_field-{self.cleaned_data["multiple_column_field"]}')
+
+                if self.cleaned_data["multiple_column_field"]:
+                    attributes.append(f'multiple_column_field-{self.cleaned_data["multiple_column_field"]}')
+
+                if self.cleaned_data['append_column_title']:
+                    attributes.append('append_column_title-1')
+
+                if self.cleaned_data["positive_colour_field"]:
+                    attributes.append(f'positive_colour_field-{self.cleaned_data["positive_colour_field"]}')
+
+                if self.cleaned_data["negative_colour_field"]:
+                    attributes.append(f'negative_colour_field-{self.cleaned_data["negative_colour_field"]}')
 
         if attributes:
             return '-'.join(attributes)
@@ -368,7 +404,6 @@ class BarChartFieldModal(QueryBuilderModalBaseMixin, FormModal):
                           'val': _attr})
 
         self.add_command({'function': 'html', 'selector': f'#{selector} span', 'html': form.cleaned_data['title']})
-        self.add_command({'function': 'save_query_builder_id_query_data'})
         self.add_command({'function': 'update_selection'})
         return self.command_response('close')
 
@@ -394,7 +429,12 @@ class BarChartFieldModal(QueryBuilderModalBaseMixin, FormModal):
                                 template='django_modals/fields/label_checkbox.html',
                                 field_class='col-6 input-group-sm'),
                         Div(
+                            FieldEx('append_column_title',
+                                    template='django_modals/fields/label_checkbox.html',
+                                    field_class='col-6 input-group-sm'),
                             FieldEx('multiple_column_field'),
+                            FieldEx('positive_colour_field'),
+                            FieldEx('negative_colour_field'),
                             css_id='multiple_columns_fields_div'),
                         css_id='filter_fields_div'),
                     css_id='annotations_fields_div')
@@ -412,22 +452,16 @@ class BarChartFieldModal(QueryBuilderModalBaseMixin, FormModal):
 class BarChartBreakdownFieldForm(TableFieldForm):
     cancel_class = 'btn-secondary modal-cancel'
 
+    def submit_button(self, css_class='btn-success modal-submit', button_text='Submit', **kwargs):
+        if isinstance(self.django_field, NUMBER_FIELDS):
+            return StrictButton(button_text, onclick=f'save_modal_{self.form_id}()', css_class=css_class, **kwargs)
+        else:
+            return super().submit_button(css_class, button_text, **kwargs)
+
 
 class BarChartBreakdownFieldModal(TableFieldModal):
     form_class = BarChartBreakdownFieldForm
-
-    def form_valid(self, form):
-        selector = self.slug['selector']
-
-        _attr = form.get_additional_attributes()
-        self.add_command({'function': 'set_attr',
-                          'selector': f'#{selector}',
-                          'attr': 'data-attr',
-                          'val': _attr})
-
-        self.add_command({'function': 'html', 'selector': f'#{selector} span', 'html': form.cleaned_data['title']})
-        self.add_command({'function': 'breakdown_update_selection'})
-        return self.command_response('close')
+    update_selection_command = 'breakdown_update_selection'
 
 
 class BarChartShowBreakdownModal(TableUtilsMixin, Modal):
@@ -471,10 +505,12 @@ class BarChartShowBreakdownModal(TableUtilsMixin, Modal):
 
         table_fields = bar_chart_report.breakdown_fields
         fields_used = set()
+        fields_map = {}
         self.process_query_results(report_builder_class=report_builder_class,
                                    table=table,
                                    base_model=base_model,
                                    fields_used=fields_used,
+                                   fields_map=fields_map,
                                    table_fields=table_fields)
         data_index = int(self.slug['data'])
         table_field = chart_fields[data_index]
