@@ -2,18 +2,20 @@ import base64
 import json
 from datetime import timedelta, datetime
 
-from crispy_forms.bootstrap import StrictButton
 from date_offset.date_offset import DateOffset
 from dateutil.relativedelta import relativedelta
 from django.apps import apps
+from django.core.exceptions import FieldError
 from django.db import ProgrammingError, DataError
 from django.db.models import Q
+from django.forms import ChoiceField
 from django.shortcuts import get_object_or_404
+from django.template.defaultfilters import title
 from django.utils.safestring import mark_safe
 from django.views.generic import TemplateView
 from django_datatables.datatables import DatatableTable
 from django_menus.menu import MenuItem
-from django_modals.forms import CrispyForm
+from django_modals.widgets.select2 import Select2
 
 from advanced_report_builder.columns import ReportBuilderDateColumn
 from advanced_report_builder.exceptions import ReportError
@@ -23,6 +25,7 @@ from advanced_report_builder.globals import NUMBER_FIELDS, ANNOTATION_VALUE_FUNC
 from advanced_report_builder.models import ReportType
 from advanced_report_builder.utils import split_slug, split_attr, get_report_builder_class
 from advanced_report_builder.variable_date import VariableDate
+from advanced_report_builder.views.helpers import QueryBuilderForm
 from advanced_report_builder.views.report import ReportBase
 from advanced_report_builder.views.report_utils_mixin import ReportUtilsMixin
 from advanced_report_builder.views.targets.utils import get_target_value
@@ -43,7 +46,7 @@ class ChartJSTable(DatatableTable):
             targets = []
             try:
                 data = self.get_table_array(self.kwargs.get('request'), self.get_query())
-            except DataError:
+            except (DataError, FieldError):
                 data = [['N/A']]
             if self.targets is not None:
                 targets = self.targets.all()
@@ -199,23 +202,53 @@ class ChartBaseView(ReportBase, ReportUtilsMixin, TemplateView):
                 data_attr = split_attr(table_field)
                 if data_attr.get('multiple_columns') == '1':
                     query = self.extra_filters(query=table.model.objects)
+
                     multiple_column_field = data_attr.get('multiple_column_field')
+                    if multiple_column_field is None:
+                        return
                     report_builder_class = self._get_report_builder_class(base_model=base_model,
                                                                           field_str=multiple_column_field,
                                                                           report_builder_class=report_builder_class)
-                    _fields = report_builder_class.default_multiple_column_fields
+                    try:
+                        _fields = report_builder_class.default_multiple_column_fields
+                    except AttributeError:
+                        return
                     default_multiple_column_fields = [multiple_column_field + '__' + x for x in _fields]
                     order_by = f'{multiple_column_field}__{report_builder_class.default_multiple_pk}'
+                    positive_colour_field = self.get_field_path(
+                        field_name=data_attr.get('positive_colour_field'),
+                        base_model=base_model,
+                        report_builder_class=report_builder_class,
+                        table=table)
+                    if positive_colour_field is not None:
+                        default_multiple_column_fields.append(positive_colour_field)
+
+                    negative_colour_field = self.get_field_path(
+                        field_name=data_attr.get('negative_colour_field'),
+                        base_model=base_model,
+                        report_builder_class=report_builder_class,
+                        table=table)
+                    if negative_colour_field is not None:
+                        default_multiple_column_fields.append(negative_colour_field)
+
                     results = query.order_by(order_by).distinct(multiple_column_field).\
                         values(multiple_column_field, *default_multiple_column_fields)
 
                     for multiple_index, result in enumerate(results):
+
                         suffix = self._set_multiple_title(database_values=result,
                                                           value_prefix=multiple_column_field,
                                                           fields=_fields,
                                                           text=report_builder_class.default_multiple_column_text)
                         extra_filter = Q((multiple_column_field, result[multiple_column_field]))
 
+                        additional_options = {}
+                        if positive_colour_field is not None:
+                            additional_options['positive_colour'] = result[positive_colour_field]
+
+                        if negative_colour_field is not None:
+                            additional_options['negative_colour'] = result[negative_colour_field]
+
                         self.get_number_field(annotations_type=self.chart_report.axis_value_type,
                                               index=f'{index}_{multiple_index}',
                                               data_attr=data_attr,
@@ -224,7 +257,8 @@ class ChartBaseView(ReportBase, ReportUtilsMixin, TemplateView):
                                               col_type_override=col_type_override,
                                               extra_filter=extra_filter,
                                               title_suffix=suffix,
-                                              multiple_index=multiple_index)
+                                              multiple_index=multiple_index,
+                                              additional_options=additional_options)
 
                 else:
                     self.get_number_field(annotations_type=self.chart_report.axis_value_type,
@@ -235,6 +269,16 @@ class ChartBaseView(ReportBase, ReportUtilsMixin, TemplateView):
                                           col_type_override=col_type_override)
         return fields
 
+    def get_field_path(self, field_name, base_model, report_builder_class, table):
+        if field_name:
+            _, _, _, field_path = self.get_field_details(
+                base_model=base_model,
+                field=field_name,
+                report_builder_class=report_builder_class,
+                table=table)
+            return field_path
+        return None
+
     # noinspection PyUnresolvedReferences
     @staticmethod
     def add_colour_offset(colour, multiple_index):
@@ -262,6 +306,7 @@ class ChartBaseView(ReportBase, ReportUtilsMixin, TemplateView):
         self.table.extra_filters = self.extra_filters
         fields = self.process_query_results(base_model=base_model, table=self.table)
         self.table.add_columns(*fields)
+
         context['show_toolbar'] = self.show_toolbar
         context['datatable'] = self.table
         context['title'] = self.get_title()
@@ -280,7 +325,7 @@ class ChartBaseView(ReportBase, ReportUtilsMixin, TemplateView):
 
         self.add_menu('button_menu', 'button_group').add_items(
             *report_menu,
-            *self.queries_menu(),
+            *self.queries_menu(report=self.report, dashboard_report=self.dashboard_report),
         )
 
     def pod_dashboard_edit_menu(self):
@@ -303,10 +348,6 @@ class ChartBaseView(ReportBase, ReportUtilsMixin, TemplateView):
     def edit_report_menu(request, chart_report_id, slug_str):
         return []
 
-    # noinspection PyMethodMayBeStatic
-    def queries_menu(self):
-        return []
-
     @staticmethod
     def get_period_divider(annotation_value_choice, start_date_type, end_date_type):
         variable_date = VariableDate()
@@ -337,7 +378,7 @@ class ChartBaseView(ReportBase, ReportUtilsMixin, TemplateView):
         return divider
 
 
-class ChartBaseFieldForm(ReportBuilderFieldUtils, CrispyForm):
+class ChartBaseFieldForm(ReportBuilderFieldUtils, QueryBuilderForm):
     cancel_class = 'btn-secondary modal-cancel'
 
     def __init__(self, *args, **kwargs):
@@ -345,15 +386,8 @@ class ChartBaseFieldForm(ReportBuilderFieldUtils, CrispyForm):
         self.col_type_override = None
         super().__init__(*args, **kwargs)
 
-    def submit_button(self, css_class='btn-success modal-submit', button_text='Submit', **kwargs):
-        if isinstance(self.django_field, NUMBER_FIELDS):
-            return StrictButton(button_text, onclick=f'save_modal_{self.form_id}()', css_class=css_class, **kwargs)
-        else:
-            return super().submit_button(css_class, button_text, **kwargs)
-
     def cancel_button(self, css_class=cancel_class, **kwargs):
-        commands = [{'function': 'save_query_builder_id_query_data'},
-                    {'function': 'close'}]
+        commands = [{'function': 'close'}]
         return self.button('Cancel', commands, css_class, **kwargs)
 
     def get_report_type_details(self):
@@ -388,3 +422,17 @@ class ChartBaseFieldForm(ReportBuilderFieldUtils, CrispyForm):
                                                            prefix=f"{prefix}{include_field}__",
                                                            title_prefix=f"{title_prefix}{include['title']} --> ",
                                                            previous_base_model=base_model)
+
+    def setup_colour_field(self, form_fields, base_model, report_builder_class, name, data_attr, label=None):
+        colour_fields = []
+        self._get_colour_fields(base_model=base_model,
+                                report_builder_class=report_builder_class,
+                                fields=colour_fields)
+        dropdown_colour_fields = [['', '----']]
+        for colour_field in colour_fields:
+            dropdown_colour_fields.append([colour_field['id'], colour_field['text']])
+
+        form_fields[name] = ChoiceField(choices=dropdown_colour_fields, required=False, widget=Select2(), label=label)
+        if name in data_attr:
+            form_fields[name].initial = data_attr.get(name)
+
