@@ -1,5 +1,4 @@
 from django.views.generic import TemplateView
-from django.views.generic import TemplateView
 from django_menus.menu import MenuItem
 from django_menus.menu import MenuMixin
 from django_modals.modals import ModelFormModal
@@ -8,11 +7,12 @@ from django_modals.widgets.select2 import Select2Multiple
 
 from advanced_report_builder.filter_query import FilterQueryMixin
 from advanced_report_builder.models import CustomReport, ReportType
-from advanced_report_builder.utils import split_slug, make_slug_str, get_report_builder_class
+from advanced_report_builder.utils import split_slug, make_slug_str
 from advanced_report_builder.views.query_modal.mixin import MultiQueryModalMixin
+from advanced_report_builder.views.report import ReportBase
 
 
-class CustomBaseView(MenuMixin, FilterQueryMixin, TemplateView):
+class CustomBaseView(ReportBase, FilterQueryMixin, TemplateView):
     report_type_slug = None
 
     def __init__(self, **kwargs):
@@ -41,18 +41,6 @@ class CustomBaseView(MenuMixin, FilterQueryMixin, TemplateView):
         context['title'] = self.get_title()
         return context
 
-    def queries_menu(self):
-        report_queries = self.report.reportquery_set.all()
-        if len(report_queries) > 1:
-            dropdown = []
-            for report_query in report_queries:
-                slug_str = make_slug_str(self.slug, overrides={f'query{self.report.id}': report_query.id})
-                dropdown.append((self.request.resolver_match.view_name,
-                                 report_query.name, {'url_kwargs': {'slug': slug_str}}))
-            return [MenuItem(menu_display='Version', no_hover=True, css_classes='btn-secondary',
-                             dropdown=dropdown)]
-        return []
-
     def setup_menu(self):
         super().setup_menu()
         if not self.show_toolbar:
@@ -67,7 +55,7 @@ class CustomBaseView(MenuMixin, FilterQueryMixin, TemplateView):
 
         self.add_menu('button_menu', 'button_group').add_items(
             *report_menu,
-            *self.queries_menu()
+            *self.queries_menu(report=self.report, dashboard_report=self.dashboard_report)
         )
 
     def pod_dashboard_edit_menu(self):
@@ -89,13 +77,10 @@ class CustomBaseView(MenuMixin, FilterQueryMixin, TemplateView):
         else:
             return self.report.name
 
-    def get_dashboard_class(self, report):
-        return None
-
     def get_report_type(self):
         if self.report_type_slug is not None:
             if self._report_type is None:
-                self._report_type =  ReportType.objects.get(slug=self.report_type_slug)
+                self._report_type = ReportType.objects.get(slug=self.report_type_slug)
             return self._report_type
         return None
 
@@ -126,10 +111,12 @@ class CustomBaseView(MenuMixin, FilterQueryMixin, TemplateView):
             return query
         return self.get_default_filter(query)
 
+
 class CustomModal(MultiQueryModalMixin, ModelFormModal):
     process = PROCESS_EDIT_DELETE
     permission_delete = PERMISSION_OFF
     model = CustomReport
+    ajax_commands = ['datatable', 'button']
 
     widgets = {'report_tags': Select2Multiple}
 
@@ -138,12 +125,9 @@ class CustomModal(MultiQueryModalMixin, ModelFormModal):
                    'notes']
 
     def form_setup(self, form, *_args, **_kwargs):
-        fields = ['name',
-                   'report_tags',
-                   'notes']
-
+        fields = [*self.form_fields]
         if self.object.id and 'report_type' in self.slug:
-            self.add_extra_queries(form=form, fields=fields, show_order_by=True)
+            self.add_extra_queries(form=form, fields=fields)
         return fields
 
     def get_report_type(self, **_kwargs):
