@@ -23,6 +23,7 @@ from advanced_report_builder.toggle import RBToggle
 from advanced_report_builder.utils import split_attr, encode_attribute, decode_attribute, get_report_builder_class
 from advanced_report_builder.views.charts_base import ChartBaseView, ChartJSTable, ChartBaseFieldForm
 from advanced_report_builder.views.modals_base import QueryBuilderModalBaseMixin, QueryBuilderModalBase
+from advanced_report_builder.views.query_modal.mixin import MultiQueryModalMixin
 
 
 class LineChartJSTable(ChartJSTable):
@@ -90,9 +91,13 @@ class LineChartView(ChartBaseView):
         context['line_chart_report'] = self.chart_report
         return context
 
-    def set_extra_number_field_kwargs(self, data_attr, options, multiple_index):
-        line_colour = data_attr.get('line_colour') or '801C70'
-        line_colour = self.add_colour_offset(line_colour, multiple_index=multiple_index)
+    def set_extra_number_field_kwargs(self, data_attr, options, multiple_index, additional_options):
+        line_colour = None
+        if additional_options is not None:
+            line_colour = additional_options.get('positive_colour')
+        if line_colour is None or line_colour == '':
+            line_colour = data_attr.get('line_colour') or '801C70'
+            line_colour = self.add_colour_offset(line_colour, multiple_index=multiple_index)
         options.update({'colour': line_colour})
 
     def edit_report_menu(self, request,  chart_report_id, slug_str):
@@ -113,11 +118,13 @@ class LineChartView(ChartBaseView):
                                          targets=targets)
 
 
-class LineChartModal(QueryBuilderModalBase):
+class LineChartModal(MultiQueryModalMixin, QueryBuilderModalBase):
     template_name = 'advanced_report_builder/charts/modal.html'
     process = PROCESS_EDIT_DELETE
     permission_delete = PERMISSION_OFF
     model = LineChartReport
+    show_order_by = False
+
     widgets = {'line_colour': ColourPickerWidget,
                'show_totals': RBToggle,
                'has_targets': RBToggle,
@@ -144,7 +151,7 @@ class LineChartModal(QueryBuilderModalBase):
             {'selector': '#div_id_targets', 'values': {'checked': 'show'}, 'default': 'hide'},
         ])
 
-        if 'data' in _kwargs:
+        if 'data' in _kwargs and len(_kwargs['data']) > 0:
             date_field = _kwargs['data'].get('date_field')
             report_type_id = _kwargs['data'].get('report_type')
             report_type = get_object_or_404(ReportType, id=report_type_id)
@@ -159,30 +166,29 @@ class LineChartModal(QueryBuilderModalBase):
                          report_type=report_type)
 
         form.fields['notes'].widget.attrs['rows'] = 3
-        self.add_query_data(form)
 
         url = reverse('advanced_report_builder:line_chart_field_modal',
                       kwargs={'slug': 'selector-99999-data-FIELD_INFO-report_type_id-REPORT_TYPE_ID'})
 
-        return ('name',
-                'notes',
-                'report_type',
-                'report_tags',
-                'axis_scale',
-                'axis_value_type',
-                'date_field',
-                FieldEx('fields',
-                        template='advanced_report_builder/select_column.html',
-                        extra_context={'select_column_url': url,
-                                       'command_prefix': ''}),
-                'x_label',
-                'y_label',
-                'show_totals',
-                'has_targets',
-                'targets',
-                FieldEx('query_data',
-                        template='advanced_report_builder/query_builder.html'),
-                )
+        fields = ['name',
+                  'notes',
+                  'report_type',
+                  'report_tags',
+                  'axis_scale',
+                  'axis_value_type',
+                  'date_field',
+                  FieldEx('fields',
+                          template='advanced_report_builder/select_column.html',
+                          extra_context={'select_column_url': url,
+                                         'command_prefix': ''}),
+                  'x_label',
+                  'y_label',
+                  'show_totals',
+                  'has_targets',
+                  'targets']
+        if self.object.id:
+            self.add_extra_queries(form=form, fields=fields)
+        return fields
 
     def select2_date_field(self, **kwargs):
         return self.get_fields_for_select2(field_type='date',
@@ -205,26 +211,36 @@ class LineChartFieldForm(ChartBaseFieldForm):
 
         self.fields['has_filter'] = BooleanField(required=False, widget=RBToggle())
         self.fields['filter'] = CharField(required=False)
-
         if data_attr.get('has_filter') == '1':
             self.fields['has_filter'].initial = True
             if 'filter' in data_attr:
                 self.fields['filter'].initial = decode_attribute(data_attr['filter'])
 
         self.fields['multiple_columns'] = BooleanField(required=False, widget=RBToggle())
+        self.fields['append_column_title'] = BooleanField(required=False, widget=RBToggle())
         report_builder_class = get_report_builder_class(model=base_model,
                                                         report_type=report_type)
-        fields = []
+
+        self.setup_colour_field(form_fields=self.fields,
+                                base_model=base_model,
+                                report_builder_class=report_builder_class,
+                                name='positive_colour_field',
+                                data_attr=data_attr,
+                                label='Line colour field')
+        multiple_column_field = []
         self._get_query_builder_foreign_key_fields(base_model=base_model,
                                                    report_builder_class=report_builder_class,
-                                                   fields=fields)
+                                                   fields=multiple_column_field)
 
-        self.fields['multiple_column_field'] = ChoiceField(choices=fields, required=False)
+        self.fields['multiple_column_field'] = ChoiceField(choices=multiple_column_field, required=False)
 
         if data_attr.get('multiple_columns') == '1':
             self.fields['multiple_columns'].initial = True
             self.fields['multiple_column_field'].initial = data_attr.get('multiple_column_field')
 
+            if data_attr.get('append_column_title') == '1':
+                self.fields['append_column_title'].initial = True
+
         super().setup_modal(*args, **kwargs)
 
     def get_additional_attributes(self):
@@ -241,7 +257,11 @@ class LineChartFieldForm(ChartBaseFieldForm):
                 attributes.append(f'filter-{b64_filter}')
             if self.cleaned_data['multiple_columns']:
                 attributes.append('multiple_columns-1')
+                if self.cleaned_data["positive_colour_field"]:
+                    attributes.append(f'positive_colour_field-{self.cleaned_data["positive_colour_field"]}')
                 attributes.append(f'multiple_column_field-{self.cleaned_data["multiple_column_field"]}')
+                if self.cleaned_data['append_column_title']:
+                    attributes.append('append_column_title-1')
 
         if attributes:
             return '-'.join(attributes)
@@ -270,7 +290,6 @@ class LineChartFieldModal(QueryBuilderModalBaseMixin, FormModal):
                           'val': _attr})
 
         self.add_command({'function': 'html', 'selector': f'#{selector} span', 'html': form.cleaned_data['title']})
-        self.add_command({'function': 'save_query_builder_id_query_data'})
         self.add_command({'function': 'update_selection'})
         return self.command_response('close')
 
@@ -295,7 +314,11 @@ class LineChartFieldModal(QueryBuilderModalBaseMixin, FormModal):
                                 template='django_modals/fields/label_checkbox.html',
                                 field_class='col-6 input-group-sm'),
                         Div(
+                            FieldEx('append_column_title',
+                                    template='django_modals/fields/label_checkbox.html',
+                                    field_class='col-6 input-group-sm'),
                             FieldEx('multiple_column_field'),
+                            FieldEx('positive_colour_field'),
                             css_id='multiple_columns_fields_div'),
                         css_id='filter_fields_div'),
                     css_id='annotations_fields_div')
