@@ -17,6 +17,7 @@ from advanced_report_builder.toggle import RBToggle
 from advanced_report_builder.utils import split_attr, encode_attribute, decode_attribute, get_report_builder_class
 from advanced_report_builder.views.charts_base import ChartBaseView, ChartBaseFieldForm
 from advanced_report_builder.views.modals_base import QueryBuilderModalBaseMixin, QueryBuilderModalBase
+from advanced_report_builder.views.query_modal.mixin import MultiQueryModalMixin
 
 
 class PieChartView(ChartBaseView):
@@ -34,9 +35,13 @@ class PieChartView(ChartBaseView):
         context['pie_chart_report'] = self.chart_report
         return context
 
-    def set_extra_number_field_kwargs(self, data_attr, options, multiple_index):
-        pie_colour = data_attr.get('pie_colour') or '801C70'
-        pie_colour = self.add_colour_offset(pie_colour, multiple_index=multiple_index)
+    def set_extra_number_field_kwargs(self, data_attr, options, multiple_index, additional_options):
+        pie_colour = None
+        if additional_options is not None:
+            pie_colour = additional_options.get('positive_colour')
+        if pie_colour is None or pie_colour == '':
+            pie_colour = data_attr.get('pie_colour') or '801C70'
+            pie_colour = self.add_colour_offset(pie_colour, multiple_index=multiple_index)
         options.update({'colour': pie_colour})
 
     def edit_report_menu(self, request, chart_report_id, slug_str):
@@ -49,12 +54,12 @@ class PieChartView(ChartBaseView):
         return None
 
 
-class PieChartModal(QueryBuilderModalBase):
+class PieChartModal(MultiQueryModalMixin, QueryBuilderModalBase):
     template_name = 'advanced_report_builder/charts/modal.html'
     process = PROCESS_EDIT_DELETE
     permission_delete = PERMISSION_OFF
     model = PieChartReport
-    widgets = {'report_tags': Select2Multiple}
+    show_order_by = False
 
     form_fields = ['name',
                    'notes',
@@ -62,29 +67,28 @@ class PieChartModal(QueryBuilderModalBase):
                    'report_tags',
                    'axis_value_type',
                    'style',
-                   'fields',
-                   ]
+                   'fields']
+
+    widgets = {'report_tags': Select2Multiple}
 
     def form_setup(self, form, *_args, **_kwargs):
-
-        self.add_query_data(form)
-
         url = reverse('advanced_report_builder:pie_chart_field_modal',
                       kwargs={'slug': 'selector-99999-data-FIELD_INFO-report_type_id-REPORT_TYPE_ID'})
         form.fields['notes'].widget.attrs['rows'] = 3
-        return ('name',
-                'notes',
-                'report_type',
-                'report_tags',
-                'axis_value_type',
-                'style',
-                FieldEx('fields',
-                        template='advanced_report_builder/select_column.html',
-                        extra_context={'select_column_url': url,
-                                       'command_prefix': ''}),
-                FieldEx('query_data',
-                        template='advanced_report_builder/query_builder.html'),
-                )
+        fields = ['name',
+                  'notes',
+                  'report_type',
+                  'report_tags',
+                  'axis_value_type',
+                  'style',
+                  FieldEx('fields',
+                          template='advanced_report_builder/select_column.html',
+                          extra_context={'select_column_url': url,
+                                         'command_prefix': ''}),
+                  ]
+        if self.object.id:
+            self.add_extra_queries(form=form, fields=fields)
+        return fields
 
 
 class PieChartFieldForm(ChartBaseFieldForm):
@@ -109,18 +113,29 @@ class PieChartFieldForm(ChartBaseFieldForm):
                 self.fields['filter'].initial = decode_attribute(data_attr['filter'])
 
         self.fields['multiple_columns'] = BooleanField(required=False, widget=RBToggle())
+        self.fields['append_column_title'] = BooleanField(required=False, widget=RBToggle())
         report_builder_class = get_report_builder_class(model=base_model,
                                                         report_type=report_type)
-        fields = []
+
+        self.setup_colour_field(form_fields=self.fields,
+                                base_model=base_model,
+                                report_builder_class=report_builder_class,
+                                name='positive_colour_field',
+                                data_attr=data_attr,
+                                label='Pie colour field')
+
+        multiple_column_field = []
         self._get_query_builder_foreign_key_fields(base_model=base_model,
                                                    report_builder_class=report_builder_class,
-                                                   fields=fields)
+                                                   fields=multiple_column_field)
 
-        self.fields['multiple_column_field'] = ChoiceField(choices=fields, required=False)
+        self.fields['multiple_column_field'] = ChoiceField(choices=multiple_column_field, required=False)
 
         if data_attr.get('multiple_columns') == '1':
             self.fields['multiple_columns'].initial = True
             self.fields['multiple_column_field'].initial = data_attr.get('multiple_column_field')
+            if data_attr.get('append_column_title') == '1':
+                self.fields['append_column_title'].initial = True
 
         super().setup_modal(*args, **kwargs)
 
@@ -138,7 +153,11 @@ class PieChartFieldForm(ChartBaseFieldForm):
                 attributes.append(f'filter-{b64_filter}')
             if self.cleaned_data['multiple_columns']:
                 attributes.append('multiple_columns-1')
+                if self.cleaned_data["positive_colour_field"]:
+                    attributes.append(f'positive_colour_field-{self.cleaned_data["positive_colour_field"]}')
                 attributes.append(f'multiple_column_field-{self.cleaned_data["multiple_column_field"]}')
+                if self.cleaned_data['append_column_title']:
+                    attributes.append('append_column_title-1')
 
         if attributes:
             return '-'.join(attributes)
@@ -167,7 +186,6 @@ class PieChartFieldModal(QueryBuilderModalBaseMixin, FormModal):
                           'val': _attr})
 
         self.add_command({'function': 'html', 'selector': f'#{selector} span', 'html': form.cleaned_data['title']})
-        self.add_command({'function': 'save_query_builder_id_query_data'})
         self.add_command({'function': 'update_selection'})
         return self.command_response('close')
 
@@ -192,7 +210,11 @@ class PieChartFieldModal(QueryBuilderModalBaseMixin, FormModal):
                                 template='django_modals/fields/label_checkbox.html',
                                 field_class='col-6 input-group-sm'),
                         Div(
+                            FieldEx('append_column_title',
+                                    template='django_modals/fields/label_checkbox.html',
+                                    field_class='col-6 input-group-sm'),
                             FieldEx('multiple_column_field'),
+                            FieldEx('positive_colour_field'),
                             css_id='multiple_columns_fields_div'),
                         css_id='filter_fields_div'),
                     css_id='annotations_fields_div')
