@@ -11,7 +11,7 @@ from advanced_report_builder.columns import ReportBuilderCurrencyPenceColumn, Re
     ReportBuilderNumberColumn
 from advanced_report_builder.field_utils import ReportBuilderFieldUtils
 from advanced_report_builder.filter_query import FilterQueryMixin
-from advanced_report_builder.globals import ANNOTATION_FUNCTIONS, ANNOTATION_CHOICE_COUNT
+from advanced_report_builder.globals import ANNOTATION_FUNCTIONS, ANNOTATION_CHOICE_COUNT, ALIGNMENT_CLASS
 from advanced_report_builder.utils import decode_attribute
 
 
@@ -21,8 +21,10 @@ class ReportUtilsMixin(ReportBuilderFieldUtils, FilterQueryMixin):
 
     def get_number_field(self, annotations_type, index, table_field, data_attr, fields, col_type_override,
                          extra_filter=None, title_suffix='', multiple_index=0, decimal_places=None,
-                         convert_currency_fields=False, totals=None, divider=None):
+                         convert_currency_fields=False, totals=None, divider=None, additional_options=None):
         field_name = table_field['field']
+        alignment_class = ALIGNMENT_CLASS.get(int(data_attr.get('alignment', 0)))
+        new_field_name = field_name
         css_class = None
         annotation_filter = None
         if annotations_type != 0:
@@ -36,6 +38,8 @@ class ReportUtilsMixin(ReportBuilderFieldUtils, FilterQueryMixin):
 
         if int(data_attr.get('display_heading', 1)) == 0:
             title = ''
+        elif int(data_attr.get('append_column_title', 0)) == 0:
+            title = title_suffix
         else:
             title = title_suffix + ' ' + table_field.get('title')
         if col_type_override:
@@ -61,19 +65,29 @@ class ReportUtilsMixin(ReportBuilderFieldUtils, FilterQueryMixin):
                     field.title = title
                 self.set_extra_number_field_kwargs(data_attr=data_attr,
                                                    options=field.options,
-                                                   multiple_index=multiple_index)
+                                                   multiple_index=multiple_index,
+                                                   additional_options=additional_options)
 
             elif annotations_type == ANNOTATION_CHOICE_COUNT:
-                new_field_name = f'{annotations_type}_{field_name}_{index}'
+                new_field_name = self.get_new_annotation_field_name(annotations_type=annotations_type,
+                                                                    field_name=field_name,
+                                                                    index=index,
+                                                                    data_attr=data_attr)
                 number_function_kwargs = {}
                 if title:
                     number_function_kwargs['title'] = title
+
+                if alignment_class != '':
+                    css_class = alignment_class
+                    number_function_kwargs['column_defs'] = {'className': css_class}
+
                 function_type = ANNOTATION_FUNCTIONS[annotations_type]
 
                 number_function_kwargs['options'] = {}
                 self.set_extra_number_field_kwargs(data_attr=data_attr,
                                                    options=number_function_kwargs['options'],
-                                                   multiple_index=multiple_index)
+                                                   multiple_index=multiple_index,
+                                                   additional_options=additional_options)
                 if annotation_filter:
                     function = function_type(raw_field_name, filter=annotation_filter)
                 else:
@@ -88,14 +102,23 @@ class ReportUtilsMixin(ReportBuilderFieldUtils, FilterQueryMixin):
                     number_function_kwargs['annotations'] = []
 
                 number_function_kwargs.update({'field': new_field_name,
-                                               'column_name': field_name})
+                                               'column_name': new_field_name})
                 field = self.number_field(**number_function_kwargs)
             else:
                 css_class = field.column_defs.get('className')
+                if css_class is None:
+                    css_class = alignment_class
+                else:
+                    css_class += ' ' + alignment_class
+                field.column_defs['className'] = css_class
+
                 if title:
                     field.title = title
                 if annotations_type != 0:
-                    new_field_name = f'{annotations_type}_{field_name}_{index}'
+                    new_field_name = self.get_new_annotation_field_name(annotations_type=annotations_type,
+                                                                        field_name=field_name,
+                                                                        index=index,
+                                                                        data_attr=data_attr)
                     function_type = ANNOTATION_FUNCTIONS[annotations_type]
                     if annotation_filter:
                         function = function_type(raw_field_name, filter=annotation_filter)
@@ -113,7 +136,8 @@ class ReportUtilsMixin(ReportBuilderFieldUtils, FilterQueryMixin):
                     field.field = new_field_name
                     self.set_extra_number_field_kwargs(data_attr=data_attr,
                                                        options=field.options,
-                                                       multiple_index=multiple_index)
+                                                       multiple_index=multiple_index,
+                                                       additional_options=additional_options)
             fields.append(field)
         else:
             number_function_kwargs = {'title': title}
@@ -124,8 +148,13 @@ class ReportUtilsMixin(ReportBuilderFieldUtils, FilterQueryMixin):
                 number_function_kwargs['options'] = {}
                 self.set_extra_number_field_kwargs(data_attr=data_attr,
                                                    options=number_function_kwargs['options'],
-                                                   multiple_index=multiple_index)
-                new_field_name = f'{annotations_type}_{field_name}_{index}'
+                                                   multiple_index=multiple_index,
+                                                   additional_options=additional_options)
+
+                new_field_name = self.get_new_annotation_field_name(annotations_type=annotations_type,
+                                                                    field_name=field_name,
+                                                                    index=index,
+                                                                    data_attr=data_attr)
                 function_type = ANNOTATION_FUNCTIONS[annotations_type]
                 if annotation_filter:
                     function = function_type(field_name, filter=annotation_filter)
@@ -149,16 +178,32 @@ class ReportUtilsMixin(ReportBuilderFieldUtils, FilterQueryMixin):
             show_total = data_attr.get('show_totals')
             if show_total == '1':
                 self.set_number_total(totals=totals,
-                                      field_name=field_name,
+                                      field_name=new_field_name,
                                       col_type_override=col_type_override,
                                       decimal_places=decimal_places,
                                       css_class=css_class)
-
         return field_name
 
-    def set_extra_number_field_kwargs(self, data_attr, options, multiple_index):
+    @staticmethod
+    def get_new_annotation_field_name(annotations_type, field_name, index, data_attr):
+        annotation_column_id = data_attr.get('annotation_column_id')
+        if annotation_column_id:
+            return decode_attribute(annotation_column_id)
+        return f'{annotations_type}_{field_name}_{index}'
+
+    def set_extra_number_field_kwargs(self, data_attr, options, multiple_index, additional_options):
         pass
 
     # noinspection PyUnusedLocal,PyMethodMayBeStatic
-    def set_number_total(self, totals, field_name, col_type_override, decimal_places, css_class):
+    def set_number_total(self, totals, field_name, col_type_override, decimal_places, css_class=''):
         totals[field_name] = {'sum': 'to_fixed', 'decimal_places': decimal_places, 'css_class': css_class}
+
+    def set_annotation_total(self, totals, field_name, col_type_override, decimal_places, css_class=''):
+        return self.set_number_total(totals, field_name, col_type_override, decimal_places, css_class)
+
+    def set_percentage_total(self, totals, field_name, denominator, numerator, decimal_places, css_class=''):
+        totals[field_name] = {'sum': 'percentage',
+                              'decimal_places': decimal_places,
+                              'denominator': denominator,
+                              'numerator': numerator,
+                              'css_class': css_class}
