@@ -9,8 +9,9 @@ from django.urls import reverse
 from django_datatables.datatables import DatatableTable
 from django_menus.menu import MenuItem
 from django_modals.fields import FieldEx
+from django_modals.form_helpers import HorizontalNoEnterHelper
 from django_modals.helper import show_modal
-from django_modals.modals import Modal
+from django_modals.modals import Modal, ModelFormModal
 from django_modals.processes import PROCESS_EDIT_DELETE, PERMISSION_OFF
 from django_modals.widgets.select2 import Select2Multiple
 from django_modals.widgets.widgets import Toggle
@@ -19,13 +20,15 @@ from advanced_report_builder.columns import ReportBuilderNumberColumn
 from advanced_report_builder.exceptions import ReportError
 from advanced_report_builder.globals import NUMBER_FIELDS, ANNOTATION_CHOICE_SUM, \
     ANNOTATION_CHOICE_AVERAGE_SUM_FROM_COUNT
-from advanced_report_builder.models import SingleValueReport, ReportType
-from advanced_report_builder.utils import get_report_builder_class
+from advanced_report_builder.models import SingleValueReport, ReportType, ReportQuery
+from advanced_report_builder.utils import get_report_builder_class, get_query_js
 from advanced_report_builder.variable_date import VariableDate
 from advanced_report_builder.views.charts_base import ChartBaseView
 from advanced_report_builder.views.datatables.modal import TableFieldModal, TableFieldForm
 from advanced_report_builder.views.datatables.utils import TableUtilsMixin
-from advanced_report_builder.views.modals_base import QueryBuilderModalBase
+from advanced_report_builder.views.helpers import QueryBuilderModelForm
+from advanced_report_builder.views.modals_base import QueryBuilderModalBase, QueryBuilderModalBaseMixin
+from advanced_report_builder.views.query_modal.mixin import MultiQueryModalMixin
 
 
 class SingleValueView(ChartBaseView):
@@ -237,13 +240,20 @@ class SingleValueView(ChartBaseView):
         context['single_value_report'] = self.chart_report
         return context
 
+    def get_breakdown_slug(self):
+        query_id = self.slug.get(f'query{self.report.pk}')
+        if query_id is None and self.dashboard_report is not None:
+            query_id = self.dashboard_report.report_query_id
+        slug = f'pk-{self.table.single_value.id}-enable_links-{self.table.enable_links}'
+        if query_id is not None:
+            slug += f'-query{self.report.pk}-{query_id}'
+        return slug
+
     def get_breakdown_url(self):
         if self.table.single_value.show_breakdown:
+            slug = self.get_breakdown_slug()
             return show_modal('advanced_report_builder:single_value_show_breakdown_modal',
-                              'pk-',
-                              self.table.single_value.id,
-                              '-enable_links-',
-                              self.table.enable_links, href=True)
+                              slug, href=True)
         return None
 
     def pod_dashboard_view_menu(self):
@@ -255,15 +265,13 @@ class SingleValueView(ChartBaseView):
                          font_awesome='fas fa-pencil-alt', css_classes=['btn-primary']),
                 *self.duplicate_menu(request=self.request, report_id=chart_report_id)]
 
-    def queries_menu(self):
-        return []
 
-
-class SingleValueModal(QueryBuilderModalBase):
+class SingleValueModal(MultiQueryModalMixin, QueryBuilderModalBase):
     template_name = 'advanced_report_builder/single_values/modal.html'
     process = PROCESS_EDIT_DELETE
     permission_delete = PERMISSION_OFF
     model = SingleValueReport
+    show_order_by = False
     widgets = {'report_tags': Select2Multiple,
                'show_breakdown': Toggle(attrs={'data-onstyle': 'success', 'data-on': 'YES', 'data-off': 'NO'})}
 
@@ -297,14 +305,17 @@ class SingleValueModal(QueryBuilderModalBase):
             {'selector': '#div_id_numerator',
              'values': {SingleValueReport.SINGLE_VALUE_TYPE_PERCENT: 'show'},
              'default': 'hide'},
-            {'selector': '#div_id_extra_query_data',
+            {'selector': '.btn-query-edit',
+             'values': {SingleValueReport.SINGLE_VALUE_TYPE_PERCENT: 'hide',
+                        SingleValueReport.SINGLE_VALUE_TYPE_PERCENT_FROM_COUNT: 'hide'},
+             'default': 'show'},
+            {'selector': '.btn-query-numerator-edit',
              'values': {SingleValueReport.SINGLE_VALUE_TYPE_PERCENT: 'show',
                         SingleValueReport.SINGLE_VALUE_TYPE_PERCENT_FROM_COUNT: 'show'},
              'default': 'hide'},
             {'selector': 'label[for=id_field]',
              'values': {SingleValueReport.SINGLE_VALUE_TYPE_PERCENT: ('html', 'Denominator field')},
              'default': ('html', 'Field')},
-
             {'selector': '#div_id_average_scale',
              'values': {SingleValueReport.SINGLE_VALUE_TYPE_AVERAGE_SUM_OVER_TIME: 'show'},
              'default': 'hide'},
@@ -320,7 +331,7 @@ class SingleValueModal(QueryBuilderModalBase):
             {'selector': '#div_id_breakdown_fields', 'values': {'checked': 'show'}, 'default': 'hide'},
         ])
 
-        if 'data' in _kwargs:
+        if 'data' in _kwargs and len(_kwargs['data']) > 0:
             field = _kwargs['data'].get('field')
             numerator = _kwargs['data'].get('numerator')
             report_type_id = _kwargs['data'].get('report_type')
@@ -342,7 +353,6 @@ class SingleValueModal(QueryBuilderModalBase):
                          selected_field_id=numerator,
                          report_type=report_type)
 
-        self.add_query_data(form, include_extra_query=True)
         form.fields['notes'].widget.attrs['rows'] = 3
         url = reverse('advanced_report_builder:single_value_field_modal',
                       kwargs={'slug': 'selector-99999-data-FIELD_INFO-report_type_id-REPORT_TYPE_ID'})
@@ -350,30 +360,33 @@ class SingleValueModal(QueryBuilderModalBase):
         range_type_choices = VariableDate.RANGE_TYPE_CHOICES
         form.fields['average_start_period'] = ChoiceField(required=False, choices=range_type_choices)
         form.fields['average_end_period'] = ChoiceField(required=False, choices=range_type_choices)
-        return ('name',
-                'notes',
-                'report_type',
-                'report_tags',
-                'single_value_type',
-                'numerator',
-                'field',
-                'average_scale',
-                'average_start_period',
-                'average_end_period',
-                'prefix',
-                FieldEx('extra_query_data',
-                        template='advanced_report_builder/query_builder.html',
-                        ),
-                'tile_colour',
-                'decimal_places',
-                'show_breakdown',
-                FieldEx('breakdown_fields',
-                        template='advanced_report_builder/select_column.html',
-                        extra_context={'select_column_url': url,
-                                       'command_prefix': ''}),
-                FieldEx('query_data',
-                        template='advanced_report_builder/query_builder.html')
-                )
+        fields = ['name',
+                  'notes',
+                  'report_type',
+                  'report_tags',
+                  'single_value_type',
+                  'numerator',
+                  'field',
+                  'average_scale',
+                  'average_start_period',
+                  'average_end_period',
+                  'prefix',
+                  'tile_colour',
+                  'decimal_places',
+                  'show_breakdown',
+                  FieldEx('breakdown_fields',
+                          template='advanced_report_builder/select_column.html',
+                          extra_context={'select_column_url': url,
+                                         'command_prefix': ''})]
+        if self.object.id:
+            self.add_extra_queries(form=form, fields=fields)
+            if self.object.single_value_type in [SingleValueReport.SINGLE_VALUE_TYPE_PERCENT,
+                                                 SingleValueReport.SINGLE_VALUE_TYPE_PERCENT_FROM_COUNT]:
+                self.add_page_command('set_css', selector='.btn-query-edit', prop='display', val='none')
+            else:
+                self.add_page_command('set_css', selector='.btn-query-numerator-edit',
+                                      prop='display', val='none')
+        return fields
 
     def select2_field(self, **kwargs):
         return self.get_fields_for_select2(field_type='number',
@@ -412,6 +425,25 @@ class SingleValueModal(QueryBuilderModalBase):
                          report_builder_class=report_builder_fields)
         return self.command_response('report_fields', data=json.dumps({'fields': fields, 'tables': tables}))
 
+    def query_menu(self):
+        edit_numerator_query_js = get_query_js('edit_numerator_query', 'query_id')
+        description_edit_menu_items = super().query_menu()
+        description_edit_menu_items.append(MenuItem(edit_numerator_query_js.replace('"', "&quot;"),
+                                                    menu_display='Edit',
+                                                    css_classes='btn btn-sm btn-outline-dark btn-query-numerator-edit',
+                                                    font_awesome='fas fa-pencil',
+                                                    link_type=MenuItem.HREF,
+                                                    hide=True))
+        return description_edit_menu_items
+
+    def button_edit_numerator_query(self, **_kwargs):
+        query_id = _kwargs['query_id'][1:]
+        report_type = self.get_report_type(**_kwargs)
+        url = reverse('advanced_report_builder:single_value_numerator_modal',
+                      kwargs={'slug': f'pk-{query_id}-'
+                                      f'report_type-{report_type}'})
+        return self.command_response('show_modal', modal=url)
+
 
 class SingleValueShowBreakdownModal(TableUtilsMixin, Modal):
     button_container_class = 'text-center'
@@ -434,10 +466,12 @@ class SingleValueShowBreakdownModal(TableUtilsMixin, Modal):
         report_builder_class = get_report_builder_class(model=base_model,
                                                         report_type=self.table_report.report_type)
         fields_used = set()
+        fields_map = {}
         self.process_query_results(report_builder_class=report_builder_class,
                                    table=table,
                                    base_model=base_model,
                                    fields_used=fields_used,
+                                   fields_map=fields_map,
                                    table_fields=table_fields)
 
         table.ajax_data = False
@@ -453,27 +487,61 @@ class SingleValueShowBreakdownModal(TableUtilsMixin, Modal):
 class SingleValueTableFieldForm(TableFieldForm):
     cancel_class = 'btn-secondary modal-cancel'
 
-    def cancel_button(self, css_class=cancel_class, **kwargs):
-        commands = [{'function': 'save_query_builder_id_query_data'},
-                    {'function': 'save_query_builder_id_extra_query_data'},
-                    {'function': 'close'}]
-        return self.button('Cancel', commands, css_class, **kwargs)
-
 
 class SingleValueTableFieldModal(TableFieldModal):
     form_class = SingleValueTableFieldForm
 
-    def form_valid(self, form):
-        selector = self.slug['selector']
 
-        _attr = form.get_additional_attributes()
-        self.add_command({'function': 'set_attr',
-                          'selector': f'#{selector}',
-                          'attr': 'data-attr',
-                          'val': _attr})
+class QueryNumeratorForm(QueryBuilderModelForm):
+    cancel_class = 'btn-secondary modal-cancel'
 
-        self.add_command({'function': 'html', 'selector': f'#{selector} span', 'html': form.cleaned_data['title']})
-        self.add_command({'function': 'save_query_builder_id_query_data'})
-        self.add_command({'function': 'save_query_builder_id_extra_query_data'})
-        self.add_command({'function': 'update_selection'})
+    class Meta:
+        model = ReportQuery
+        fields = ['name',
+                  'query',
+                  'extra_query']
+
+    def setup_modal(self, *args, **kwargs):
+        self.fields['extra_query'].label = 'Numerator'
+        super().setup_modal(*args, **kwargs)
+
+
+class QueryNumeratorModal(QueryBuilderModalBaseMixin, ModelFormModal):
+    model = ReportQuery
+    process = PROCESS_EDIT_DELETE
+    permission_delete = PERMISSION_OFF
+    form_class = QueryNumeratorForm
+    helper_class = HorizontalNoEnterHelper
+    ajax_commands = ['datatable', 'button']
+
+    template_name = 'advanced_report_builder/extra_query_modal.html'
+    no_header_x = True
+
+    def __init__(self, *args, **kwargs):
+        self._base_and_class = None
+        super().__init__(*args, **kwargs)
+
+    def post_save(self, created, form):
+        self.add_command({'function': 'save_query_builder_id_extra_query'})
         return self.command_response('close')
+
+    def form_setup(self, form, *_args, **_kwargs):
+        fields = ['name',
+                  FieldEx('query', template='advanced_report_builder/query_builder.html'),
+                  FieldEx('extra_query', template='advanced_report_builder/query_builder.html')]
+
+        return fields
+
+    def ajax_get_query_builder_fields(self, **kwargs):
+        field_auto_id = kwargs['field_auto_id']
+        report_type_id = self.slug['report_type']
+        query_builder_filters = self.get_query_builder_report_type_field(report_type_id=report_type_id)
+        return self.command_response(f'query_builder_{field_auto_id}', data=json.dumps(query_builder_filters))
+
+    def get_report_builder_base_and_class(self):
+        if self._base_and_class is None:
+            self._base_and_class = self.get_report_builder_class(report_type_id=self.slug['report_type'])
+        return self._base_and_class
+
+    def get_report_type(self):
+        return self.slug['report_type']
